# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import logging

from jsonsimpleconfig import JscComments, JscData, JscSection


###############################################################################
# Class                                                                       #
###############################################################################

class JscParser:
    """
    JscParser
    """

    def __init__(self):
        self.__jsc_data = None
        self.__current_section = None
        self.__current_section_data = None

    def __new_section(self, section_name):
        if section_name[0] == '[':
            section_name = section_name.lstrip('[')
        if section_name[-1] == ']':
            section_name = section_name.rstrip(']')
        if section_name != JscSection.GLOBAL_SECTION_NAME:
            section_name = section_name.strip()
            if section_name[0] != '"':
                section_name = '"' + section_name
            if section_name[-1] != '"':
                section_name = section_name + '"'

        self.__current_section = section_name
        self.__current_section_data = '{'

    def __end_section(self):
        self.__current_section_data += '}'
        if self.__jsc_data is None:
            self.__jsc_data = JscData()

        section_name = self.__current_section
        json_string = self.__current_section_data
        self.__current_section_data = self.__current_section = None
        self.__jsc_data.add_section_json_string(section_name, json_string)

    def __parse_line(self, line):
        line = JscComments.strip_comments(line)
        line = line.strip()
        if line:
            if line[0] == '[' and line[-1] == ']':
                if self.__current_section is not None and \
                        self.__current_section_data is not None:
                    self.__end_section()
                self.__new_section(line)

            else:
                if self.__current_section is None and \
                        self.__current_section_data is None:
                    self.__new_section(JscSection.GLOBAL_SECTION_NAME)
                if self.__current_section_data != '{':
                    self.__current_section_data += ','
                self.__current_section_data += line

    def parse_file(self, jsc_file_path, parse_line_by_line=False) -> JscData:
        """
        parse_file
        :param jsc_file_path:
        :param parse_line_by_line:
        :return:
        """
        self.__jsc_data = None
        try:
            with open(jsc_file_path) as jsc_file:

                if jsc_file.readable():
                    if parse_line_by_line:
                        for line in jsc_file:
                            self.__parse_line(line)
                    else:
                        jsc_text = JscComments.strip_comments(jsc_file.read())
                        lines = jsc_text.splitlines()
                        for line in lines:
                            self.__parse_line(line)
            self.__end_section()
            jsc_file.close()

        except Exception as exception:
            self.__jsc_data = None
            logging.debug(exception)

        return self.__jsc_data

    def parse_string(self, jsc_string) -> JscData:
        """
        parse_string
        :param jsc_string:
        :return:
        """
        self.__jsc_data = None
        try:
            jsc_text = JscComments.strip_comments(jsc_string)
            lines = jsc_text.splitlines()
            for line in lines:
                self.__parse_line(line)
            self.__end_section()

        except Exception as exception:
            self.__jsc_data = None
            logging.debug(exception)

        return self.__jsc_data

###############################################################################
#                                End of file                                  #
###############################################################################



#! /bin/sh
# -*- coding: utf-8 -*-
""":"
exec python3 $0 ${1+"$@"}
"""
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__doc__ = "Convert JSON Simple Config file to JSON"
__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import argparse
import logging
import signal
import sys

from jsonsimpleconfig import Jsc2JsonHelper

###############################################################################
# Module Variable(s)                                                          #
###############################################################################

VERSION_STRING = "0.0.1"
APPLICATION_NAME_STRING = "JSC converter to JSON"


###############################################################################
# Module                                                                      #
###############################################################################

def parameters():
    """
    parameters
    :return:
    """
    parser = argparse.ArgumentParser(description=APPLICATION_NAME_STRING)
    parser.add_argument('-v', '--version', action='version', version=APPLICATION_NAME_STRING + " - " + VERSION_STRING)
    parser.add_argument('-i', '--in', type=argparse.FileType('r'), help='Input file path (JSC file)', required=True)
    parser.add_argument('-o', '--out', type=argparse.FileType('w'), help='Output file path (JSON file)', required=False)
    logging_level_choices = {
        'CRITICAL': logging.CRITICAL,
        'ERROR': logging.ERROR,
        'WARNING': logging.WARNING,
        'INFO': logging.INFO,
        'DEBUG': logging.DEBUG
    }
    parser.add_argument('-ll', '--logging_level', dest="loggingLevel", choices=logging_level_choices.keys(),
                        help='Output log level', required=False)
    args, _ = parser.parse_known_args()

    if vars(args)['loggingLevel'] is None:
        level = logging.CRITICAL
    else:
        level = logging_level_choices.get(vars(args)['loggingLevel'], logging.CRITICAL)
    logging.basicConfig(format='[%(asctime)s][%(levelname)-8s] [%(module)-20s] - %(message)s',
                        datefmt='%Y.%m.%d %H:%M.%S', level=level)

    jsc_file = (vars(args)['in']).name

    if vars(args)['out'] is None:
        json_file = jsc_file + ".json"
    else:
        json_file = (vars(args)['out']).name

    return {'loggingLevel': (vars(args)['loggingLevel']), 'json_file': json_file, 'jsc_file': jsc_file}


def main(argv=sys.argv):
    """
    main
    :param argv:
    """
    signal.signal(signal.SIGINT, handler)
    args = parameters()

    if 'loggingLevel' in args and args['loggingLevel'] == 'DEBUG':
        Jsc2JsonHelper.convert(args['jsc_file'], args['json_file'])
    else:
        try:
            Jsc2JsonHelper.convert(args['jsc_file'], args['json_file'])
        except Exception as exception:
            print('Error!')
            logging.debug(exception)


def handler(signum, frame):
    """
    handler
    :param signum:
    :param frame:
    """
    sys.exit()


# Execute main function
if __name__ == '__main__':
    main()
    sys.exit()

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import collections
import json
import logging

from jsonsimpleconfig import JscData, JscSection


###############################################################################
# Class                                                                       #
###############################################################################

class JsonExtractor:

    def __init__(self, jsc_data, options=None):
        self.__jsc_data = jsc_data
        self.__json_data = {}
        self.__file = None
        self.__version = '1.0'
        self.__options = options

    def get_version(self):
        return self.__version

    def get_json(self):
        return self.__json_data

    def __extract_section_to_json(self, section_name):
        branches = section_name.split(JscSection.SECTION_GLUE)
        json_data = self.__json_data
        if isinstance(branches, collections.Iterable):
            for branch in branches:
                branch = branch.strip().strip('"')
                if branch != JscSection.GLOBAL_SECTION_NAME:
                    if branch not in json_data:
                        json_data[branch] = {}
                    json_data = json_data[branch]
        section = self.__jsc_data.get_section(section_name)
        if isinstance(section, collections.Iterable):
            for item in section:
                json_data[item] = section[item]

    def extract_data_to_json(self):
        if JscSection.GLOBAL_SECTION_NAME in self.__jsc_data.get_section_names():
            self.__extract_section_to_json(JscSection.GLOBAL_SECTION_NAME)
        for section_name in self.__jsc_data.get_section_names():
            if section_name != JscSection.GLOBAL_SECTION_NAME:
                self.__extract_section_to_json(section_name)

    def extract_to_file(self, file_path):
        if self.__jsc_data is not None and isinstance(self.__jsc_data, JscData):
            self.extract_data_to_json()
            try:
                json_string = json.dumps(self.__json_data, ensure_ascii=False, indent=4)

            except json.JSONDecodeError as error:
                json_string = None
                logging.debug(error)

            if json_string is not None:
                self.__file = open(file_path, 'w', encoding='utf-8')
                if self.__file is not None:
                    self.__file.write(json_string)
                    self.__file.close()

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import json
import logging
import sys

from jsonsimpleconfig import JscData


###############################################################################
# Class                                                                       #
###############################################################################

class Json:
    """
    Json
    """

    def __init__(self, file):
        self.__file = file

    def parse(self):
        """
        parse
        :return:
        """

        try:
            json_data = open(self.__file)
            data = json.load(json_data)
            status = None
            json_data.close()

        except json.JSONDecodeError as exception:
            data = None
            status = exception.msg

        except Exception as exception:
            data = None
            status = sys.exc_info()
            logging.debug(exception)

        return {'data': data, 'status': status}

    @staticmethod
    def parse_file_json(json_file_path) -> JscData:
        """
        parse_file_json
        :param json_file_path: path to the JSON file
        :return: JSC Data
        """
        try:
            with open(json_file_path) as json_file:
                json_data = json.load(json_file)
                jsc_data = JscData.from_json(json_data)
        except Exception as exception:
            jsc_data = None
            logging.debug(exception)

        return jsc_data

    @staticmethod
    def parse_string_json(json_string) -> JscData:
        """
        parse_string_json
        :param json_string: JSON string
        :return: JSC Data
        """
        try:
            json_data = json.loads(json_string)
            jsc_data = JscData.from_json(json_data)
        except Exception as exception:
            jsc_data = None
            logging.debug(exception)

        return jsc_data

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import logging
import os
import os.path
from string import Template

import jsonsimpleconfig


###############################################################################
# Class                                                                       #
###############################################################################

class JscHeader:

    def __init__(self, options={'defaultHeader': True,
                                'allowCustomHeader': False,
                                'customHeaderFilePath': '/etc/jsonsimpleconfig/header.template'}):
        self.__options = options

    def __header_template(self, file_path) -> Template:
        header = None
        try:
            if os.path.isfile(file_path):
                filein = open(file_path)
                if filein:
                    jsc_template_file = Template(filein.read())
                    if self.__options.get('headerTemplateData'):
                        header = jsc_template_file.substitute(self.__options.get('headerTemplateData'))
        except OSError as error:
            logging.debug(error)
        return header

    def str(self) -> str:
        header = file_path = None

        if self.__options.get('allowCustomHeader') and \
                ("customHeaderFilePath" in self.__options) and \
                os.path.isfile(self.__options.get('customHeaderFilePath')):
            file_path = self.__options.get('customHeaderFilePath')

        elif self.__options.get('defaultHeader'):
            path = os.path.abspath(jsonsimpleconfig.__file__)
            path = os.path.dirname(path)
            file_path = os.path.join(path, 'jscresources/defaultHeader.template')

        if file_path is not None:
            header = self.__header_template(file_path)

        return header

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."


###############################################################################
# Class                                                                       #
###############################################################################


class JscSection:
    """
    The JSC section data container.
    """
    SECTION_GLUE = "."
    GLOBAL_SECTION_NAME = "__GLOBAL__"

    def __init__(self, name=GLOBAL_SECTION_NAME):
        self.__section = list()
        self.__section.append(name)

    def __append(self, name):
        self.__section.append('"' + name + '"')

    def go_up(self, name):
        """
        Move level up for current section

        Parameters
        ----------
        name : string
            Section name
        """
        if len(self.__section) == 1 and JscSection.GLOBAL_SECTION_NAME in self.__section:
            self.__section = list()
        self.__append(name)

    def go_down(self):
        """
        Move level down for current section
        """
        if len(self.__section) == 1:
            self.__section = list()
            self.__section.append(JscSection.GLOBAL_SECTION_NAME)
        else:
            del self.__section[-1]

    def str(self):
        """
        Get JSC data section as string

        Returns
        -------
        string
            JSC data section as string.
        """
        if len(self.__section) == 1 and JscSection.GLOBAL_SECTION_NAME in self.__section:
            section_name = None
        else:
            section_name = JscSection.SECTION_GLUE.join(self.__section)
        return section_name

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import logging

from jsonsimpleconfig import JscParser


###############################################################################
# Class                                                                       #
###############################################################################

class Jsc:
    class JsonSimpleConfig:
        def __init__(self, jsc_file):
            self.__jsc_file = jsc_file
            logging.debug('Parsing JSC file: %s', jsc_file)
            jsc_parser = JscParser()
            self.__jsc_data = jsc_parser.parse_file(jsc_file)

        __jsc_data = None

        def get(self):
            return self.__jsc_data

        def get_file(self):
            return self.__jsc_file

    __instance = None

    @staticmethod
    def set_logging_level(level):
        logging.basicConfig(format='[%(asctime)s][%(levelname)-8s] [%(module)-20s] - %(message)s',
                            datefmt='%Y.%m.%d %H:%M.%S', level=level)

    @staticmethod
    def set_debug_logging_level():
        Jsc.set_logging_level(logging.DEBUG)

    @staticmethod
    def get(jsc_file, refresh=False):
        if refresh or \
                not Jsc.__instance or \
                Jsc.__instance.get_file() != jsc_file:
            Jsc.__instance = Jsc.JsonSimpleConfig(jsc_file)

        return Jsc.__instance.get()

    @staticmethod
    def gets(jsc_data):
        jsc_parser = JscParser()
        return jsc_parser.parse_string(jsc_data)

    @staticmethod
    def get_value(jsc_file, section, key):
        jsc_data = Jsc.get(jsc_file, True)
        if jsc_data is not None:
            return jsc_data.get_value(section, key)
        return None

###############################################################################
#                                End of file                                  #
###############################################################################



"""
JSON Simple Config

The simple idea to prepare and manage configuration for your application.
"""

# * ********************************************************************* *
# *   Copyright (C) 2020 by xmz                                           *
# * ********************************************************************* *

###############################################################################
# Import(s)                                                                   #
###############################################################################

from .jsccommon import JscComments, JscCommon
from .jscdata import JscData, JscSection
from .jscparser import Json, JscParser
from .jscextractor import JscExtractor, JscHeader, JsonExtractor
from .jschelper import Jsc, Json2JscHelper, Jsc2JsonHelper, JscConfig, JscConfigManaged as JscDataFromFile


###############################################################################
# Module                                                                      #
###############################################################################

def load(jsc_file, refresh=False):
    """
    load
    :param jsc_file:
    :param refresh:
    :return:
    """
    return Jsc.get(jsc_file, refresh)


def loads(jsc_string):
    """
    loads
    :param jsc_string:
    :return:
    """
    return Jsc.gets(jsc_string)


def load_json(json_file):
    """
    load_json
    :param json_file:
    :return:
    """
    return Json.parse_file_json(json_file)


def loads_json(json_string):
    """
    loads_json
    :param json_string:
    :return:
    """
    return Json.parse_string_json(json_string)


__all__ = (
    'JscCommon',
    'JscComments',
    'Json2JscHelper',
    'Jsc2JsonHelper',
    'Json',
    'JscParser',
    'JscData',
    'JscSection',
    'JscExtractor',
    'JscExtractor',
    'JscHeader',
    'JsonExtractor',
    'JscDataFromFile',
    'JscConfig',
    'Jsc',
)

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2020 by xmz                                           *
# * ********************************************************************* *

"""
JSC Config class - JSC Config singleton to keep current configuration data
"""

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

from jsonsimpleconfig import JscParser, JscData


###############################################################################
# Class                                                                       #
###############################################################################

class JscConfig:
    """JSC Config class."""
    _jsc_configs = ['/etc/jsc/main.jsc']
    __instance = None
    __jsc_data = None

    def init(self):
        """
        Initial load config from _jsc_configs files.
        """
        for jsc_file in self._jsc_configs:
            self.merge_jsc_file(jsc_file)

    def __new__(cls):
        if JscConfig.__instance is None:
            JscConfig.__instance = object.__new__(cls)
            JscConfig.__instance.init()
        return JscConfig.__instance

    @staticmethod
    def get():
        """
        Get the current configuration instance.
        """
        return JscConfig()

    @staticmethod
    def delete():
        """
        Clean current instance and allow to create new initialization for configuration.
        """
        if JscConfig.__instance is not None:
            JscConfig.__instance = None

    def get_data(self):
        """
        Returns JSC data stored for current configuration.

        Returns
        -------
        dict
            The JSC data stored at the current config
        """
        return self.__jsc_data

    def merge_jsc_file(self, jsc_file):
        """
        Merge your JSC file to current configuration.

        Parameters
        ----------
        jsc_file : string
            JSC file path
        """
        jsc_parser = JscParser()
        self.merge_jsc(jsc_parser.parse_file(jsc_file))

    def merge_jsc(self, jsc_data):
        """
        Merge your JSC data to current configuration.

        Parameters
        ----------
        jsc_data : JscData
            JSC data to merge to current configuration
        """
        if jsc_data is not None and isinstance(jsc_data, JscData):
            if self.__jsc_data is not None and isinstance(self.__jsc_data, JscData):
                self.__jsc_data.merge(jsc_data)
            else:
                self.__jsc_data = jsc_data

    def get_value(self, section_name, key, default=None):
        """
        Get value for provided section and key.
        For 'Global' variable section name should be None.

        Parameters
        ----------
        section_name : string
            Section name
        key : string
            Key name
        default : dict
            Default value if key does not exist

        Returns
        -------
        dict
            The value of the section key.
        """
        if self.__jsc_data is not None and isinstance(self.__jsc_data, JscData):
            return self.__jsc_data.get_value(section_name, key, default)

        return None

    def get_section(self, section_name, default=None):
        """
        Get full section branches.

        Parameters
        ----------
        section_name : string
            Section name
        default : dict
            Default value if section does not exist

        Returns
        -------
        dict
            The section branch structure.
        """
        if self.__jsc_data is not None and isinstance(self.__jsc_data, JscData):
            data = self.__jsc_data.get_section(section_name)
            if data is None:
                data = default
            return data
        return None

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import collections
import json
import os

from jsonsimpleconfig import JscData, JscSection, JscCommon
from .jsc_header import JscHeader


###############################################################################
# Class                                                                       #
###############################################################################

class JscExtractor:

    def __init__(self, jsc_data, options=None):
        self.__jsc_data = jsc_data
        self.__file = None
        self.__version = '1.0'
        self.__options = options

    def get_version(self):
        return self.__version

    def __extract_section_to_file(self, section_name):
        section = self.__jsc_data.get_section(section_name)
        if isinstance(section, collections.Iterable):
            for item in section:
                self.__file.write(json.dumps({item: section[item]}).strip().lstrip("{").rstrip("}") + os.linesep)
            self.__file.write(os.linesep)

    def __extract_data_to_file(self):
        if JscSection.GLOBAL_SECTION_NAME in self.__jsc_data.get_section_names():
            self.__extract_section_to_file(JscSection.GLOBAL_SECTION_NAME)
        for section_name in self.__jsc_data.get_section_names():
            if section_name != JscSection.GLOBAL_SECTION_NAME:
                self.__file.write('[' + section_name + ']' + os.linesep)
                self.__extract_section_to_file(section_name)

    def extract_to_file(self, file_path):
        if self.__jsc_data is not None and isinstance(self.__jsc_data, JscData):
            self.__file = open(file_path, 'w', encoding='utf-8')
            if self.__file is not None:
                jsc_header = JscHeader({
                    'defaultHeader': True,
                    'allowCustomHeader': True,
                    'customHeaderFilePath': '/etc/jsonsimpleconfig/header.template',
                    'headerTemplateData':
                        {
                            'JscExtractor_version': self.__version,
                            'Jsc_timestamp': JscCommon.get_timestamp()
                        }
                }).str()
                if jsc_header is not None:
                    self.__file.write(jsc_header)
                self.__extract_data_to_file()
                self.__file.close()

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2020 by xmz                                           *
# * ********************************************************************* *
"""
JSON Simple Config Setup
"""
__author__ = "Marcin Zelek"
__email__ = "marcin.zelek@gmail.com"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."
__license__ = "MIT"
__version__ = "0.3"

###############################################################################
# Import(s)                                                                   #
###############################################################################

import os

from setuptools import setup

###############################################################################
# Module                                                                      #
###############################################################################

DESCRIPTION = 'The simple idea to prepare configuration for your application.'


def read(fname):
    """test"""
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


try:
    LONG_DESCRIPTION = read('README.rst')
except IOError:
    LONG_DESCRIPTION = DESCRIPTION

setup(
    name='jsonsimpleconfig',
    version=__version__,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    keywords="json configuration jsc simple config",
    author=__author__,
    author_email=__email__,
    license=__license__,
    url='https://github.com/xmzxmz/jsonsimpleconfig',
    package_data={'': ['jscresources/*.template']},
    include_package_data=True,
    packages=['jsonsimpleconfig',
              'jsonsimpleconfig.jsccommon',
              'jsonsimpleconfig.jscdata',
              'jsonsimpleconfig.jscextractor',
              'jsonsimpleconfig.jschelper',
              'jsonsimpleconfig.jscparser'],
    entry_points={
        'console_scripts':
            [
                'json2jsc = jsonsimpleconfig.json2jsc:main',
                'jsc2json = jsonsimpleconfig.jsc2json:main',
                'jsc_value = jsonsimpleconfig.jsc_value:main',
                'jsc_print = jsonsimpleconfig.jsc_print:main',
            ],
    },
    zip_safe=False
)

###############################################################################
#                                End of file                                  #
###############################################################################



# -*- coding: utf-8 -*-
# * ********************************************************************* *
# *   Copyright (C) 2018 by xmz                                           *
# * ********************************************************************* *

__author__ = "Marcin Zelek (marcin.zelek@gmail.com)"
__copyright__ = "Copyright (C) xmz. All Rights Reserved."

###############################################################################
# Import(s)                                                                   #
###############################################################################

import json
import logging
import os
import sys

from .jsc_section import JscSection


###############################################################################
# Class                                                                       #
###############################################################################

class JscData:
    """
    The JSC data container.
    """

    def __init__(self):
        self.__section = JscSection()
        self.__jsc = {}

    def __section_from_json(self, section_data):
        if isinstance(section_data, dict):
            keys = list(section_data.keys())
            if not keys:
                self.__section.go_down()
            else:
                for key, value in section_data.items():
                    if isinstance(value, dict):
                        self.__section.go_up(key)
                        self.__section_from_json(value)
                    else:
                        section_name = self.__section.str()
                        if section_name is None:
                            section_name = JscSection.GLOBAL_SECTION_NAME

                        if section_name not in self.__jsc:
                            self.__jsc[section_name] = {}
                        self.__jsc[section_name][key] = value

                    if key == keys[-1]:
                        self.__section.go_down()

    def add_section_data(self, section_name, section_data):
        """
        Put data for JSC tree section

        Parameters
        ----------
        section_name : string
            Section name
        section_data : dict
            Section data
        """
        if section_name[0] == '[':
            section_name = section_name.lstrip('[')
        if section_name[-1] == ']':
            section_name = section_name.rstrip(']')
        for key, value in section_data.items():
            if section_name not in self.__jsc:
                self.__jsc[section_name] = {}
            self.__jsc[section_name][key] = value

    def add_section_json_string(self, section_name, section_json_string):
        """
        Put data for JSC tree section using JSON format
        If JSON is correct will be added as JSC data branch

        Parameters
        ----------
        section_name : string
            Section name
        section_json_string : string
            Section data as JSON string
        """
        try:
            section_data = json.loads(section_json_string)
            self.add_section_data(section_name, section_data)

        except json.JSONDecodeError as exception:
            logging.error(exception.msg)
            logging.error(sys.exc_info())
            logging.error(section_json_string)
        except Exception as exception:
            logging.error(sys.exc_info())
            logging.debug(exception)

    def get_value(self, section_name, key, default=None) -> dict:
        """
        Get value for provided section and key
        For 'Global' variable section name should be None

        Parameters
        ----------
        section_name : string
            Section name
        key : string
            Key name
        default : dict
            Default value if key does not exist

        Returns
        -------
        dict
            The value of the section key.
        """
        value = default
        section_data = self.get_section(section_name)
        if section_data and key in section_data:
            value = section_data.get(key)
        return value

    def get_section(self, section_name=None) -> dict:
        """
        Get full section branches

        Parameters
        ----------
        section_name : string
            Section name

        Returns
        -------
        dict
            The section branch structure.
        """
        if section_name is None or section_name == '':
            section_name = JscSection.GLOBAL_SECTION_NAME
        elif section_name != JscSection.GLOBAL_SECTION_NAME:
            section_name = section_name.strip()
            if section_name[0] != '"':
                section_name = '"' + section_name
            if section_name[-1] != '"':
                section_name = section_name + '"'
        if section_name in self.get_section_names():
            return self.__jsc[section_name]
        return None

    def get_section_names(self):
        """
        Get all sections

        Returns
        -------
        dict
            Return list of the sections names or None if JSC is not defined.
        """
        if self.__jsc is not None:
            return self.__jsc.keys()
        return None

    def merge(self, jsc_data):
        """
        Merge another JSC data into current data

        Parameters
        ----------
        jsc_data : JscData
            JSC data from another source
        """
        if jsc_data is not None and isinstance(jsc_data, JscData):
            section_names = jsc_data.get_section_names()
            if section_names is not None:
                self.__merge_section_names(jsc_data, section_names)

    def __merge_section_names(self, jsc_data, section_names):
        for section_name in section_names:
            section_data = jsc_data.get_section(section_name)
            if isinstance(section_data, dict):
                for key in section_data:
                    value = section_data.get(key)
                    if section_name not in self.__jsc:
                        self.__jsc[section_name] = {}
                    self.__jsc[section_name][key] = value

    def str(self) -> str:
        """
        Get JSC data structure as string

        Returns
        -------
        string
            JSC data as string.
        """
        jsc_data_string = os.linesep
        section_names = self.get_section_names()
        if section_names is not None:
            for section_name in section_names:
                section_print = "* Section"
                if section_name == JscSection.GLOBAL_SECTION_NAME:
                    section_print += " (Global):"
                else:
                    section_print += " - {}:".format(section_name)
                jsc_data_string += section_print + os.linesep
                section_data = self.get_section(section_name)
                if isinstance(section_data, dict):
                    for key in section_data:
                        value = section_data.get(key)
                        jsc_data_string += '*** [' + str(key) + '] : [' + str(value) + ']' + os.linesep

        return jsc_data_string

    def str_html(self) -> str:
        """
        Get JSC data structure as string with new line as <br> for HTML format

        Returns
        -------
        string
            HTML format of JSC data.
        """
        return self.str().replace(os.linesep, '<br>' + os.linesep)

    def print(self):
        """
        Prints JSC string data
        """
        print(self.str())

    def print_html(self):
        """
        Prints JSC data as HTML format
        """
        print(self.str_html())

    def convert_from_json(self, json_data):
        """
        Converts JSC data section from JSON

        Parameters
        ----------
        json_data : dict
            JSON data to convert as dict
        """
        self.__section_from_json(json_data)

    @staticmethod
    def from_json(json_data):
        """
        Get JSC data from JSON

        Parameters
        ----------
        json_data : dict
            JSON data to convert as dict

        Returns
        -------
        string
            JSC data from provided JSON.
        """
        jsc_data = JscData()
        jsc_data.convert_from_json(json_data)
        return jsc_data

###############################################################################
#                                End of file                                  #
###############################################################################
